var cli = require('sk/node/cli'),
	sys = require('sys'),
	Queue = require('sk/core/queue').Queue,
	fs = require('fs');


var CliInputQueue = Queue.extend({
	'override add': function(ops)
	{
		if(typeof ops == 'function') return this._super(ops);
		
		var self = this;
		
		this._super(function()
		{
			process.stdout.write(ops.label);
			
			cli.next(function(argument)
			{
				ops.callback(argument);
				self.next();
			});
		});
	}
});

function nextArg(callback)
{
	return function()
	{
		cli.next(function(argument)
		{
			callback(argument);
		})
	}
}

function printResponse(response, message)
{
	if(response.result || response.message)
	{
		if(typeof message == 'function') return message();
		
		console.log(message.green);
	}
	else
	if(response.errors)
	{
		for(var i = response.errors.length; i--;)
		{
			var error = response.errors[i];
			console.log(error.message.red);
		}
	}
}

exports.init = function(controller){

	var q = new CliInputQueue(true),
		availablePlugins = {};
	
	
	controller.getAvailablePlugins(function(response)
	{
		var plugins = response.result || [];
		
		
		plugins.forEach(function (plugin)
		{
			availablePlugins[plugin.ns] = plugin;	
		});
		
	});
	
	function showAvailablePlugins()
	{
		for(var pluginNS in availablePlugins)
		{
			console.log(pluginNS.magenta);
		}
	}
	
	function inputCredentials(){
		
		q.add({
			label: 'username: ',
			callback: controller.getMethod('user')
		});
		
		q.add({
			label: 'password: ',
			callback: controller.getMethod('pass')
		});
		
		q.add(function(){
			// console.log('validating auth credentials...'.blue);
			
			controller.login(function(success){
				if (success) {
					//not initialize the commands
					initCommands();
				}
				else {
					cli.confirm('Authentication credentials are incorrect, signup?', function(){
						controller.signup(function(success)
						{
							if(success)
							{
								initCommands();
							}
							else
							{
								console.log('Unable to signup. The username probably already exists.'.red);
								inputCredentials();
							}
						});
					}, function(){
						inputCredentials();
					})
				}
				
				q.next();
			});
		});
	}
	
	
	function initCommands(){
		console.log(('Logged in as ' + controller.user()+'.').green);
					
					
		cli.on('create', function()
		{
			
			cli.next({
				app: function()
				{
					cli.next(function(appName)
					{
						// console.log(('Creating a new application "'+appName+'"...').blue);
				
						controller.createApp(appName, function(response)
						{
							if(response.result)
							{
								console.log('Successfuly saved app'.green);
							}
							else
							{
								console.log('Unable to save app'.red);
							}
						});
					});
				}
			});
		});
		
		
		cli.on('remove', function()
		{
			cli.next({
				app: function()
				{
					cli.next(function(appName)
					{
						cli.confirm('Are you sure you want to remove the app "'+appName+'"?', function()
						{
							// console.log(('Removing app "'+appName+'"...').blue);
							
							controller.removeApp(appName, function(response)
							{
								printResponse(response, 'Successfuly removed app');
							});
						});
					})
				}
			})
		})
		
		cli.on(['start','restart'], function ()
		{
			cli.next(function (appName)
			{
				controller.startApp(appName, cli.args.length ? cli.args.shift() : null, function(success)
				{
					if(success)
					{
						console.log('Successfuly started app'.green);
					}
					else
					{
						console.log('Unable to start app.'.red);
					}
				});
			});
		});
		
		cli.on('npm', function()
		{
			cli.next({
				install: function()
				{
					cli.next(function (packageName)
					{
						controller.installNPMPackage(packageName, function()
						{
							console.log('installed NPM Package'.green);
						});
					});
				}
			})
		})
		
		cli.on('whitelist', function()
		{
			cli.next({
				add: function()
				{
					cli.next(function(url)
					{
						cli.next(function(app)
						{
							controller.whitelistAdd(url, app, function()
							{
								console.log('added url to the whitelist'.green);
							});
						});
					});
				},
				remove: function()
				{
					cli.next(function(url)
					{
						cli.next(function(app)
						{
							controller.whitelistRemove(url, app, function()
							{
								console.log('removed url from the whitelist'.green);
							});
						});
					});
				}
			})
		})
		
		cli.on('stop', function ()
		{
			cli.next(function (appName)
			{
				controller.stopApp(appName, cli.args.length ? cli.args.shift() : null, function(success)
				{
					if(success)
					{
						console.log('Successfuly stopped app'.green);
					}
					else
					{
						console.log('Unable to stop app.'.red);
					}
				});
			});
		});
		
		
		cli.on('help', function ()
		{
			cli.openHelp(__dirname + '/help.txt');
		});
		
		
		cli.on('install', function()
		{
			cli.next(function(pluginNS)
			{
				cli.next(function(appName)
				{
					function installPlugin(availablePlugin, onInstalled)
					{
						var promptQueue = new CliInputQueue(true);

						cli.confirm('Would you like to install "'+availablePlugin.ns+'"?', function()
						{
							function addPlugin(params)
							{
								controller.addPlugin(availablePlugin.ns, appName, params, function(success)
								{
									if(!success)
									{
										console.log(('Unable to install "' +availablePlugin.ns+ '"').red);
									}

									if(onInstalled) onInstalled();
								});
							}

							var params = {};


							//some plugins require parameters, so prepare the input
							if(availablePlugin.params)
							{
								function addParamInput(param, paramInput)
								{
									promptQueue.add({
										label: paramInput.name + ': ',
										callback: function(value)
										{
											params[param] = value;
										}
									})
								}
								for(var param in availablePlugin.params)
								{
									var paramInput = availablePlugin.params[param];

									addParamInput(param, paramInput)
								}

								promptQueue.add(function()
								{
									addPlugin(params);
									promptQueue.next();
								});
							}
							else
							{
								addPlugin();
							}
						},
						function()
						{
							q.next();
						})
					}

				
					function fail()
					{
						console.error('Plugin "'+pluginNS+'" doesn\'t exist. please select from the following:');

						return showAvailablePlugins();
					}

					//wildcard = add multiple
					if(pluginNS.indexOf('*') > -1)
					{
						var prefix = pluginNS.substr(0,pluginNS.indexOf('*')),
						toInstall = [];

						for(var ns in availablePlugins)
						{
							if(ns.indexOf(prefix) == 0)
							{
								toInstall.push(availablePlugins[ns]);
							}
						}

						if(!toInstall.length)
						{
							fail();
						}
						else
						{

							toInstall.forEach(function (availablePlugin)
							{
								q.add(function()
								{
									installPlugin(availablePlugin, q.getMethod('next'));
								});
							})
							
						}
					}
					else
					{
						var availablePlugin = availablePlugins[pluginNS];

						if(!availablePlugin) return fail();

						q.add(function()
						{
							installPlugin(availablePlugin, q.getMethod('next'));
						})
					}
					
					
					q.add(function()
					{
						console.log('Finished installing plugins.'.green);
						q.next();
					})

				});


			});
		});
		
		cli.on('uninstall', function()
		{
			cli.next(function(pluginName)
			{
				cli.next(function(appName)
				{
					cli.confirm('Are you sure you want to remove the plugin "'+pluginName+'" from "'+appName+'"?', function()
					{
						controller.removePlugin(pluginName, appName, function()
						{
							console.log("Successfuly removed plugin".green);
						});	
					});

				});
			});
		});
		
		
		cli.on('list', function()
		{
			cli.next({
				apps: function()
				{
					// console.log('Fetching apps...'.blue);
					
					controller.getApps(function(apps)
					{
						if(apps.result)
						{
							apps.result.forEach(function (app)
							{
								console.log(app.name.blue + ': ' + (app.running.length ? app.running.join(',').green : 'none'.red) );
							});
							
							// console.log('Done fetching apps.'.green);
						}
						else
						if(apps.errors)
						{
							console.log(apps.errors)
							console.log('cannot fetch apps'.red);
						}
					});
				},
				$all: function(appName)
				{
					cli.next({
						plugins: function()
						{
							// console.log(('Listing "'+appName+'" plugins...').blue);
							
							controller.getAppPlugins(appName, function(response)
							{
								printResponse(response, function()
								{
									response.result.forEach(function(service)
									{
										console.log(service.ns.magenta);
									});
									
									// console.log('Done listing plugins'.green);
								});
								
							})
						},
						whitelist: function()
						{
							controller.getWhitelist(appName, function (response)
							{
								printResponse(response, function()
								{
									response.result.forEach(function(url)
									{
										console.log(url.magenta);
									});
								})
							});
						}
					});
				}
			});
		});
	}
	
	if (!controller.user()) {
		inputCredentials();
	}
	else
	{
		initCommands();
	}
	
}
